/*
 * Picviz - Parallel coordinates ploter
 * Copyright (C) 2008 Sebastien Tricaud <toady@gscore.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * $Id$
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include <picviz.h>
#include <cairo.h>

#include "../props/color.h"

cairo_t *cr;

static void draw_text(double x, double y, char *text)
{
	cairo_set_source_rgb(cr, 0, 0, 0);
	cairo_select_font_face (cr, "Sans", CAIRO_FONT_SLANT_NORMAL,
			CAIRO_FONT_WEIGHT_BOLD);
	cairo_set_font_size (cr, 10.0);

	cairo_move_to (cr, x, y);
	cairo_show_text (cr, text);
	cairo_stroke(cr);
}

void draw_line(PicvizImage *image _U_, PcvID axis_id, PicvizLine *line, PicvizAxisPlot *axisplot1, PicvizAxisPlot *axisplot2, PcvWidth x1, PcvHeight y1, PcvWidth x2, PcvHeight y2)
{
	char *color = NULL;
	double line_width;

	switch(engine.heatline_algo) {
		case 0: /* Default */
			color = picviz_properties_get(axisplot1->props, "color");
			break;
		case 1: /* Virus */
			color = picviz_properties_get(line->props, "color");
			break;
	}

	if (!color) color = picviz_properties_get(line->props, "color");

	cairo_set_source_rgb(cr,
			picviz_color_extract_r(color),
			picviz_color_extract_g(color),
			picviz_color_extract_b(color));

	line_width = atof(picviz_properties_get(line->props, "penwidth"));
	if (line_width) {
		cairo_set_line_width (cr, atof(picviz_properties_get(line->props, "penwidth"))*5);
	}
	cairo_move_to(cr, (double)x1, (double) y1);
	cairo_line_to(cr, (double)x2, (double) y2);
	cairo_stroke(cr);

	if (engine.display_raw_data) {
		if (axis_id == 1) {
			draw_text(x1, y1, axisplot1->strval);
			draw_text(x2, y2, axisplot2->strval);
		} else {
			draw_text(x2, y2, axisplot2->strval);
		}
	}

}

cairo_status_t outstdout(void *closure _U_, unsigned char *data, unsigned int length)
{
	if ( fwrite(data, 1, length, stdout) != length )
		return CAIRO_STATUS_WRITE_ERROR;

	return CAIRO_STATUS_SUCCESS;
}

void output(PicvizImage *image, char *arg _U_)
{
	PicvizAxis *axis;
	PicvizLine *line;
	cairo_surface_t *surface;

	surface = cairo_image_surface_create(CAIRO_FORMAT_RGB24,
			(int)image->width, (int)image->height);

	cr = cairo_create (surface);

	cairo_rectangle (cr, 0, 0, (int)image->width, (int)image->height);
	cairo_set_source_rgb (cr, 1, 1, 1);
	cairo_fill (cr);

	cairo_set_source_rgb(cr, 0, 0, 0);
	cairo_set_line_width (cr, 0.5);

	/* Draw each axis and header titles */
	llist_for_each_entry(axis, &image->axes, list) {
		/* Draw vertical lines */
		cairo_move_to(cr, (double) axis->xpos, (double)image->header_height);
		cairo_line_to(cr, (double) axis->xpos, (double)image->height);
	}

	cairo_stroke(cr);

	if (image->header_height) {
		//cairo_set_source_rgb(cr, 0, 0, 0);
		cairo_move_to(cr, (double)0, (double)image->header_height);
		cairo_line_to(cr, (double)image->width, (double)image->header_height);
		llist_for_each_entry(axis, &image->axes, list) {
			cairo_text_extents_t extents;
			char *label = picviz_properties_get(axis->props, "label");
			cairo_text_extents(cr, label, &extents);
			draw_text(axis->xpos - (extents.width/2), 10, label);
		}
		cairo_stroke(cr);
	}

	llist_for_each_entry(line, &image->lines, list) {
		if ( ! line->hidden ) {
			picviz_line_draw(image, line, draw_line);
		}
	}

	cairo_surface_write_to_png_stream(surface, (cairo_write_func_t)outstdout, NULL);

	cairo_destroy (cr);
	cairo_surface_destroy (surface);

}


